#!/bin/bash

# This script can be run to compile Qt Jambi on the local machine:
# ./build-qtjambi.sh
# but this:
# * may bundle a system Qt which depends on too new libraries for some machines,
# * will only build either 32-bit or 64-bit binaries, not both.
#
# The most reliable way to build the binaries is to run the following 2 commands
# on a 64-bit Fedora or RHEL/CentOS/SL machine:
# ./build-qtjambi.sh epel-6-i386
# ./build-qtjambi.sh epel-6-x86_64

# if a parameter is given, build in a mock chroot
if [ -n "$1" ] ; then
  echo '--- fetching binary Qt SDK ---'
  case "$1" in
    *x86_64*)
      QTSDK=Qt_SDK_Lin64_offline_v1_1_4_en.run
      ;;
    *)
      QTSDK=Qt_SDK_Lin32_offline_v1_1_4_en.run
      ;;
  esac
  if [ ! -f $QTSDK ] ; then
    wget ftp://ftp.informatik.hu-berlin.de/pub/Mirrors/ftp.troll.no/QT/qtsdk/$QTSDK
    chmod +x $QTSDK
  fi
  if [ ! -f qtjambi-src-4.7-beta2.tar.gz ] ; then
    wget http://downloads.sourceforge.net/project/qtjambi/4.7.0-beta2/qtjambi-src-4.7-beta2.tar.gz
  fi
  echo '--- initializing chroot ---'
  mock -r "$1" --init || exit 1
  echo '--- installing java-1.6.0-openjdk-devel --- '
  mock -r "$1" --install java-1.6.0-openjdk-devel || exit 1
  echo '--- installing xorg-x11-server-Xvfb --- '
  mock -r "$1" --install xorg-x11-server-Xvfb || exit 1
  echo '--- installing xorg-x11-xauth --- '
  mock -r "$1" --install xorg-x11-xauth || exit 1
  echo '--- installing ant --- '
  mock -r "$1" --install ant || exit 1
  echo '--- installing ant-trax --- '
  mock -r "$1" --install ant-trax || exit 1
  echo '--- installing mesa-libGL-devel --- '
  mock -r "$1" --install mesa-libGL-devel || exit 1
  echo '--- installing mesa-libGLU-devel --- '
  mock -r "$1" --install mesa-libGLU-devel || exit 1
  echo '--- mkdir -p /builddir/Concise/progs/qtjambi ---'
  mock -r "$1" --unpriv --chroot 'mkdir -p /builddir/Concise/progs/qtjambi' || exit 1
  echo '--- mkdir -p /builddir/Concise/progs/text ---'
  mock -r "$1" --unpriv --chroot 'mkdir -p /builddir/Concise/progs/text' || exit 1
  echo '--- copying build-qtjambi.sh in ---'
  mock -r "$1" --copyin build-qtjambi.sh /builddir/Concise/progs/qtjambi || exit 1
  echo '--- copying '"$QTSDK"' in ---'
  mock -r "$1" --copyin $QTSDK /builddir/Concise/progs/qtjambi || exit 1
  echo '--- copying auto_installations_script.qs in ---'
  mock -r "$1" --copyin auto_installations_script.qs /builddir/Concise/progs/qtjambi || exit 1
  echo '--- copying qtjambi-src-4.7-beta2.tar.gz in ---'
  mock -r "$1" --copyin qtjambi-src-4.7-beta2.tar.gz /builddir/Concise/progs/qtjambi || exit 1
  echo '--- copying DynGenPar sources in ---'
  mock -r "$1" --copyin ../text/dyngenpar /builddir/Concise/progs/text/dyngenpar || exit 1
  echo '--- rm -rf /builddir/Concise/progs/text/dyngenpar/build ---'
  mock -r "$1" --chroot 'rm -rf /builddir/Concise/progs/text/dyngenpar/build' || exit 1
  echo '--- rm -rf /builddir/Concise/progs/text/dyngenpar/qtjambi/{bin,concise,cpp,java/*.class,lib,target} ---'
  mock -r "$1" --chroot 'rm -rf /builddir/Concise/progs/text/dyngenpar/qtjambi/{bin,concise,cpp,java/*.class,lib,target}' || exit 1
  echo '--- rm -rf /builddir/Concise/progs/text/dyngenpar/qtjambi/scripts/{\$*,extjars,lib,*.o,*.log,.preprocessed.tmp,Makefile} ---'
  mock -r "$1" --chroot 'rm -rf /builddir/Concise/progs/text/dyngenpar/qtjambi/scripts/{\$*,extjars,lib,*.o,*.log,.preprocessed.tmp,Makefile}' || exit 1
  echo '--- chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/build-qtjambi.sh ---'
  mock -r "$1" --chroot 'chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/build-qtjambi.sh' || exit 1
  echo '--- chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/'"$QTSDK"' ---'
  mock -r "$1" --chroot 'chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/'"$QTSDK" || exit 1
  echo '--- chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/auto_installations_script.qs ---'
  mock -r "$1" --chroot 'chown -R --reference=/builddir/Concise/progs/qtjambi /builddir/Concise/progs/qtjambi/auto_installations_script.qs' || exit 1
  echo '--- chown -R --reference=/builddir/Concise/progs/text /builddir/Concise/progs/text/dyngenpar ---'
  mock -r "$1" --chroot 'chown -R --reference=/builddir/Concise/progs/text /builddir/Concise/progs/text/dyngenpar' || exit 1
  echo '--- sed -i -e "s/^USE_SYSTEM_QT=.*$/USE_SYSTEM_QT=no/g" build-qtjambi.sh ---'
  mock -r "$1" --unpriv --cwd=/builddir/Concise/progs/qtjambi --chroot 'sed -i -e "s/^USE_SYSTEM_QT=.*$/USE_SYSTEM_QT=no/g" build-qtjambi.sh' || exit 1
  echo '--- ./build-qtjambi.sh ---'
  mock -r "$1" --unpriv --cwd=/builddir/Concise/progs/qtjambi --chroot ./build-qtjambi.sh || exit 1
  echo '--- ls /builddir/Concise/progs/qtjambi/qt-jambi-qtjambi-4_7/qtjambi-linux*-gcc-*.jar /builddir/Concise/progs/text/dyngenpar/qtjambi/target/dyngenpar-linux*-gcc-*.jar ---'
  RESULTS=`mock -r "$1" --unpriv --chroot 'ls /builddir/Concise/progs/qtjambi/qt-jambi-qtjambi-4_7/qtjambi-linux*-gcc-*.jar /builddir/Concise/progs/text/dyngenpar/qtjambi/target/dyngenpar-linux*-gcc-*.jar'`
  echo '--- copying results out ---'
  mock -r "$1" --copyout $RESULTS . || exit 1
  echo '--- cleaning chroot ---'
  mock -r "$1" --clean || exit 1
  exit 0
fi

# use the system Qt if it is new enough and the -dev/-devel package is installed
USE_SYSTEM_QT=yes
# set this to no if you really want to build your own Qt (may take hours!)
USE_BINARY_QTSDK=yes

# find JDK
if [ -z $JAVA_HOME ] ; then
  JAVAC=`which javac`
  JAVAC=`readlink -f "$JAVAC"`
  if [ ! -f "$JAVAC" ] ; then
    echo error: JAVA_HOME not set and javac not found in PATH
    exit 1
  fi
  JAVA_HOME=`dirname "$JAVAC"`
  JAVA_HOME=`dirname "$JAVA_HOME"`
  export JAVA_HOME
fi
echo using Java in "$JAVA_HOME"

# install a Qt 4.7.x (or use system Qt 4.[678].x) to build Qt Jambi against
QTJAMBIDIR=`pwd`
if [ $USE_SYSTEM_QT = yes ] ; then
  if pkg-config QtCore --atleast-version=4.6 ; then
    :
  else
    echo warning: system Qt too old
    USE_SYSTEM_QT=no
  fi
fi
if [ $USE_SYSTEM_QT = yes ] ; then
  echo using system Qt
  QTVERSION=`pkg-config --modversion QtCore`
  unset QTDIR
  # make sure the unsuffixed Qt 4 executables are found in PATH
  if [ `uname -m` = x86_64 -a -e /usr/lib64/libQtCore.so ] ; then
    PATH="/usr/lib64/qt4/bin:$PATH"
  else
    PATH="/usr/lib/qt4/bin:$PATH"
  fi
  export PATH
elif [ $USE_BINARY_QTSDK = yes ] ; then
  echo using binary Qt SDK
  if [ `uname -m` = x86_64 ] ; then
    QTSDK=Qt_SDK_Lin64_offline_v1_1_4_en.run
  else
    QTSDK=Qt_SDK_Lin32_offline_v1_1_4_en.run
  fi
  if [ ! -f $QTSDK ] ; then
    wget ftp://ftp.informatik.hu-berlin.de/pub/Mirrors/ftp.troll.no/QT/qtsdk/$QTSDK
    chmod +x $QTSDK
  fi
  if [ ! -d qt-sdk-bin ] ; then
    sed -i -e 's#^var installerTargetDirectory=".*"\;$#var installerTargetDirectory="'"$QTJAMBIDIR"'/qt-sdk-bin"\;#g' auto_installations_script.qs
    xvfb-run ./$QTSDK --script auto_installations_script.qs
  fi
  QTDIR=$QTJAMBIDIR/qt-sdk-bin/Desktop/Qt/474/gcc
  QTVERSION=4.7.4
else
  echo using Qt source tarball
  echo warning: this will take very long to build
  echo warning: this configuration has not been tested yet
  if [ ! -f qt-everywhere-opensource-src-4.7.1.tar.gz ] ; then
    wget http://get.qt.nokia.com/qt/source/qt-everywhere-opensource-src-4.7.1.tar.gz
  fi
  if [ ! -d qt-everywhere-opensource-src-4.7.1 ] ; then
    tar xzf qt-everywhere-opensource-src-4.7.1.tar.gz
  fi
  QTDIR=$QTJAMBIDIR/qt-bin-4.7.1
  if [ ! -d qt-bin-4.7.1 ] ; then
    cd qt-everywhere-opensource-src-4.7.1
    ./configure -opensource -confirm-license -prefix $QTDIR
    make -j4
    make install
    cd ..
  fi
  QTVERSION=4.7.1
fi
export QTVERSION
export QTDIR
if [ -z $QTDIR ] ; then
  echo using system Qt "$QTVERSION"
else
  echo using Qt "$QTVERSION" in "$QTDIR"
fi

# build Qt Jambi
if [ ! -f qtjambi-src-4.7-beta2.tar.gz ] ; then
  wget http://downloads.sourceforge.net/project/qtjambi/4.7.0-beta2/qtjambi-src-4.7-beta2.tar.gz
fi
if [ ! -d qt-jambi-qtjambi-4_7 ] ; then
  tar xzf qtjambi-src-4.7-beta2.tar.gz
  cd qt-jambi-qtjambi-4_7
  # work around broken version check
  echo "qtjambi.version = $QTVERSION" >version.properties
  # set buildpath.properties
  if [ `uname -m` = x86_64 ] ; then
    sed -i -e 's!^qtjambi\.java\.library\.path *=.*$!qtjambi.java.library.path       = /usr/lib64!g' buildpath.properties
  fi
  if [ $USE_SYSTEM_QT = yes ] ; then
    sed -i -e 's!^qtjambi\.phonon\.pluginsdir *=.*$!qtjambi.phonon.pluginsdir       = /usr/lib/kde4!g' buildpath.properties
    if [ `uname -m` = x86_64 -a -e /usr/lib64/libQtCore.so ] ; then
      sed -i -e 's!/usr/lib!/usr/lib64!g' buildpath.properties
      sed -i -e 's!/usr/lib6464!/usr/lib64!g' buildpath.properties
    else
      sed -i -e 's!/usr/lib64!/usr/lib!g' buildpath.properties
    fi
    if [ -d /usr/include/QtCore/ ] ; then
      sed -i -e 's!^qt.includedir *=.*$!qt.includedir                   = /usr/include!g' buildpath.properties
    fi
    # prevent <sys/types.h> from producing an unwanted definition of ushort
    sed -i -e 's!#define QT_NO_STL!#define _GNU_SOURCE\n#include <sys/types.h>\n\n#define QT_NO_STL!g' generator/qtjambi_masterinclude.h
    # fix Phonon includes
    sed -i -e '/\/\/TODO: whole phonon system/d' -e's!#include <phonon>!#include <phonon/abstractaudiooutput.h>\n#include <phonon/abstractmediastream.h>\n#include <phonon/abstractvideooutput.h>\n#include <phonon/addoninterface.h>\n#include <phonon/audiodataoutput.h>\n#include <phonon/audiodataoutputinterface.h>\n#include <phonon/audiooutput.h>\n#include <phonon/audiooutputinterface.h>\n#include <phonon/backendcapabilities.h>\n#include <phonon/backendinterface.h>\n#include <phonon/effect.h>\n#include <phonon/effectinterface.h>\n#include <phonon/effectparameter.h>\n#include <phonon/effectwidget.h>\n#include <phonon/globalconfig.h>\n#include <phonon/mediacontroller.h>\n#include <phonon/medianode.h>\n#include <phonon/mediaobject.h>\n#include <phonon/mediaobjectinterface.h>\n#include <phonon/mediasource.h>\n#include <phonon/objectdescription.h>\n#include <phonon/objectdescriptionmodel.h>\n#include <phonon/path.h>\n#include <phonon/phonondefs.h>\n#include <phonon/phononnamespace.h>\n#include <phonon/platformplugin.h>\n#include <phonon/pulsesupport.h>\n#include <phonon/seekslider.h>\n#include <phonon/streaminterface.h>\n#include <phonon/videoplayer.h>\n#include <phonon/videowidget.h>\n#include <phonon/videowidgetinterface.h>\n#include <phonon/volumefadereffect.h>\n#include <phonon/volumefaderinterface.h>\n#include <phonon/volumeslider.h>\n!g' generator/qtjambi_masterinclude.h
    # reject available{AV,Video}CaptureDevices, videoCaptureDevice and MediaSource(const Phonon::ObjectDescription &)
    # wrap VideoCaptureDevice
    sed -i -e 's!<rejection class="\*" function-name="availableAudioCaptureDevices" />!<rejection class="*" function-name="availableAudioCaptureDevices" />\n    <rejection class="*" function-name="availableVideoCaptureDevices" />\n    <rejection class="*" function-name="availableAVCaptureDevices" />\n    <rejection class="*" function-name="videoCaptureDevice" />!g' -e 's!<value-type name="Phonon::MediaSource" java-name="MediaSource">!<value-type name="Phonon::MediaSource" java-name="MediaSource">\n        <modify-function signature="MediaSource(const Phonon::ObjectDescription \&amp;)" remove="all" />!g' -e 's!<value-type name="Phonon::AudioCaptureDevice"  java-name="AudioCaptureDevice" />!<value-type name="Phonon::AudioCaptureDevice"  java-name="AudioCaptureDevice" />\n     <value-type name="Phonon::VideoCaptureDevice"  java-name="VideoCaptureDevice" />!g' generator/xmlmerge/typesystem_phonon-common.xml
    # fix up VideoCaptureDevice
    tail -n +`grep -n '<value-type name="Phonon::AudioCaptureDevice">' generator/xmlmerge/typesystem_phonon-java.xml | sed -e 's/:.*$//g'` generator/xmlmerge/typesystem_phonon-java.xml >tmp.txt
    sed -i -e 's#    <namespace-type name="Phonon">#'"$(head -n +`grep -n '</value-type>' tmp.txt | sed -e 's/:.*$//g'` tmp.txt | tr '\n' '#' | sed -e 's/Audio/Video/g' -e 's/#/\\n/g')"'\n    <namespace-type name="Phonon">#g' generator/xmlmerge/typesystem_phonon-java.xml
    rm -f tmp.txt
    sed -i -e 's/^qtjambi\.phonon\.kdephonon *=.*$/qtjambi.phonon.kdephonon        = true/g' buildpath.properties
    # Qt 4.8 fixes
    sed -i -e 's/\(^.*\)toString\(.*\))\(.*$\)/\1toString\2)\3\n\1toCurrencyString\2)\3\n\1toCurrencyString\2,QString)\3/g' -e 's!    <reject-enum-value name="Qt_4_7"/>!    <reject-enum-value name="Qt_4_7"/>\n    <reject-enum-value name="Qt_4_8"/>!g' -e 's#    <modify-function signature="QCoreApplication(int &amp;, char  \*\*)">#    <modify-function signature="QCoreApplication(int \&amp;, char  **, int)">\n      <access modifier="private"/>\n      <modify-argument index="3">\n        <remove-argument/> <!-- internal, always use the default -->\n        <conversion-rule class="shell">\n          // nothing\n        </conversion-rule>\n        <conversion-rule class="native">\n          int %out = QCoreApplication::ApplicationFlags;\n        </conversion-rule>\n      </modify-argument>\n    </modify-function>\n    <modify-function signature="QCoreApplication(int \&amp;, char  **)">#g' generator/xmlmerge/typesystem_core-common.xml
    sed -i -e 's`    <modify-function signature="QApplication(int &amp;, char  \*\*, int)">`    <modify-function signature="QApplication(int \&amp;, char  **, int)">\n      <modify-argument index="3">\n        <remove-argument/> <!-- internal, always use the default -->\n        <conversion-rule class="shell">\n          // nothing\n        </conversion-rule>\n        <conversion-rule class="native">\n#if QT_VERSION \&gt;= 0x040800\n          int %out = QApplication::ApplicationFlags;\n#else\n          int %out = QT_VERSION;\n#endif\n        </conversion-rule>\n      </modify-argument>`g' -e 's!  <value-type name="QPixmap" delete-in-main-thread="yes">!  <value-type name="QPixmap" delete-in-main-thread="yes">\n    <modify-function signature="swap(QPixmap \&amp;)" access="non-final"/>!g' generator/xmlmerge/typesystem_gui-common.xml
  else
    sed -i -e 's/^qtjambi\.phonon\.kdephonon *=.*$/qtjambi.phonon.kdephonon        = false/g' buildpath.properties
  fi
  MAKEOPTS="-j4"
  LC_ALL="C"
  if [ -z $QTDIR ] ; then
    unset LD_LIBRARY_PATH
  else
    LD_LIBRARY_PATH=$QTDIR/lib
  fi
  export MAKEOPTS
  export LC_ALL
  export LD_LIBRARY_PATH
  ant
  cd ..
fi
JAMBIDIR=$QTJAMBIDIR/qt-jambi-qtjambi-4_7
export JAMBIDIR
echo Qt Jambi now in "$JAMBIDIR"

# now build the DynGenPar bindings
DYNGENPARDIR=`dirname "$QTJAMBIDIR"`/text/dyngenpar
export DYNGENPARDIR
PATH="$JAMBIDIR/generator:$PATH"
export PATH
echo building bindings for "$DYNGENPARDIR"
cd "$DYNGENPARDIR"/qtjambi/scripts
ant

# and finally build and run the test program
cd "$DYNGENPARDIR"/qtjambi/java
./testit.sh
cd "$QTJAMBIDIR"
